package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.diagbot.dto.OutputInfo;
import com.diagbot.dto.Response;
import com.diagbot.enums.CacheKeyEnum;
import com.diagbot.facade.QcClientFacade;
import com.diagbot.facade.str.*;
import com.diagbot.service.impl.ColumnServiceImpl;
import com.diagbot.vo.QueryVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class ColumnFacade extends ColumnServiceImpl {
    @Autowired
    private QcClientFacade qcClientFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    /**
     * 解析
     * @param queryVo
     */
    public void analyseRec(QueryVo queryVo){
        try {
            Response<OutputInfo> resp=qcClientFacade.analyseField(queryVo);
            if(resp.getData()!=null && resp.getData().getPageData()!=null){
                Map<String, Object> pageData=resp.getData().getPageData();
                pageData.forEach((key,value)->{
                    List<Map<String, Object>> listMap=(List<Map<String,Object>>)resp.getData().getPageData().get(key);
                    Map<String, String> columnMap=this.getColumn(CacheKeyEnum.getName(key));

                    Map<String, Object> entityMap=new HashMap<String, Object>();
                    listMap.forEach(s->{
                        columnMap.forEach((k, v) -> {
                            entityMap.put(v,s.get(k));
                        });
                        //数据解析
                        analysisData(key,entityMap,Long.valueOf(queryVo.getHospitalId()),JSON.toJSONString(s));
                    });
                });
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("数据解析","", JSON.toJSONString(queryVo),"",e.getMessage());
        }

    }

    /**
     * 数据解析
     * @param modelName
     * @param entityMap
     * @param hospitalId
     */
    private void analysisData(String modelName,Map<String, Object> entityMap,Long hospitalId,String columnMap){
        try {
            if("入院记录".equals(modelName)){
                aStrAdmissionNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("首次病程录".equals(modelName)){
                aStrFirstRecordFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("输血后效果评价".equals(modelName)){
                aStrBloodResultFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("输血/血制品病程记录".equals(modelName)){
                aStrBloodTransfusionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("转入记录".equals(modelName)){
                aStrTransferInNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("转出记录".equals(modelName)){
                aStrTransferOutNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("病危通知书".equals(modelName)){
                aStrIllCriticallyFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("病重通知书".equals(modelName)){
                aStrIllSeriouslFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("疑难病例讨论记录".equals(modelName)){
                aStrDifficultCaseFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("死亡记录".equals(modelName)){
                aStrDeathNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("死亡病例讨论记录".equals(modelName)){
                aStrDeathDiscussionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("查房记录".equals(modelName)){
                aStrWardRecordFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("术后首次病程及谈话记录".equals(modelName)){
                aStrOperativeFirstRecord.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("术前讨论、术前小结".equals(modelName)){
                aStrPreoperativeDiscussionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("手术记录".equals(modelName)){
                aStrOperativeNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("抢救记录".equals(modelName)){
                aStrRescueNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("危急值记录".equals(modelName)){
                aStrCrisisNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("出院小结".equals(modelName)){
                aStrLeaveHospitalFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("会诊申请单".equals(modelName)){
                aStrConsultationApplyFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("会诊结果单".equals(modelName)){
                aStrConsultationResultFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("会诊单".equals(modelName)){
                aStrConsultationNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if ("会诊记录".equals(modelName)){
                aStrConsultationRecordFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            } else if("阶段小结".equals(modelName)){
                aStrPeriodConclusionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }
            log.info(modelName+"---- 解析完成！");
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("数据解析异常", "",JSON.toJSONString(columnMap),"",e.getMessage());
        }
    }

    @Autowired
    private AStrAdmissionNoteFacade aStrAdmissionNoteFacade;
    @Autowired
    private AStrFirstRecordFacade aStrFirstRecordFacade;
    @Autowired
    private AStrPeriodConclusionFacade aStrPeriodConclusionFacade;
    @Autowired
    private AStrBloodResultFacade aStrBloodResultFacade;
    @Autowired
    private AStrBloodTransfusionFacade aStrBloodTransfusionFacade;
    @Autowired
    private AStrTransferInNoteFacade aStrTransferInNoteFacade;
    @Autowired
    private AStrTransferOutNoteFacade aStrTransferOutNoteFacade;
    @Autowired
    private AStrIllSeriouslFacade aStrIllSeriouslFacade;
    @Autowired
    private AStrIllCriticallyFacade aStrIllCriticallyFacade;
    @Autowired
    private AStrDifficultCaseFacade aStrDifficultCaseFacade;
    @Autowired
    private AStrDeathNoteFacade aStrDeathNoteFacade;
    @Autowired
    private AStrDeathDiscussionFacade aStrDeathDiscussionFacade;
    @Autowired
    private AStrWardRecordFacade aStrWardRecordFacade;
    @Autowired
    private AStrOperativeFirstRecord aStrOperativeFirstRecord;
    @Autowired
    private AStrPreoperativeDiscussionFacade aStrPreoperativeDiscussionFacade;
    @Autowired
    private AStrOperativeNoteFacade aStrOperativeNoteFacade;
    @Autowired
    private AStrRescueNoteFacade aStrRescueNoteFacade;
    @Autowired
    private AStrCrisisNoteFacade aStrCrisisNoteFacade;
    @Autowired
    private AStrLeaveHospitalFacade aStrLeaveHospitalFacade;
    @Autowired
    private AStrConsultationApplyFacade aStrConsultationApplyFacade;
    @Autowired
    private AStrConsultationResultFacade aStrConsultationResultFacade;
    @Autowired
    private AStrConsultationNoteFacade aStrConsultationNoteFacade;
    @Autowired
    private AStrConsultationRecordFacade aStrConsultationRecordFacade;
}
