package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrConsultationApply;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrConsultationApplyServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrConsultationApplyVO;
import com.diagbot.vo.str.StrConsultationApplyVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrConsultationApplyFacade extends StrConsultationApplyServiceImpl {

    @Autowired
    @Qualifier("strConsultationApplyServiceImpl")
    private StrConsultationApplyServiceImpl strConsultationApplyService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strConsultationApply");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    //会诊申请单
    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrConsultationApply strConsultationApply=new StrConsultationApply();
        try {
            AStrConsultationApplyVO strConsultationApplyVO=(AStrConsultationApplyVO) MapUtil.mapToObject(entityMap, AStrConsultationApplyVO.class);
            BeanUtils.copyProperties(strConsultationApplyVO,strConsultationApply);
            strConsultationApply.setHospitalId(hospitalId);//医院编码

            strConsultationApply.setAuditDate(DateUtil.parseDateTime(strConsultationApplyVO.getAuditDate()));//审核时间
            strConsultationApply.setRecDate(DateUtil.parseDateTime(strConsultationApplyVO.getRecDate()));//记录时间
            strConsultationApply.setBehospitalDate(DateUtil.parseDateTime(strConsultationApplyVO.getBehospitalDate()));//入院日期
            strConsultationApply.setRecordDate(DateUtil.parseDateTime(strConsultationApplyVO.getRecordDate()));//病历日期
            strConsultationApply.setApplyDate(DateUtil.parseDateTime(strConsultationApplyVO.getApplyDate()));//申请日期
            strConsultationApply.setConsultationDate(DateUtil.parseDateTime(strConsultationApplyVO.getConsultationDate()));//会诊时间
            strConsultationApply.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(strConsultationApply)){
                strConsultationApply.setGmtCreate(new Date());//记录创建时间
                this.save(strConsultationApply);
            }else{
                strConsultationApply.setGmtModified(new Date());//记录修改时间
                this.update(strConsultationApply,new QueryWrapper<StrConsultationApply>()
                        .eq("rec_id", strConsultationApply.getRecId())
                        .eq("hospital_id", strConsultationApply.getHospitalId())
                        .eq("behospital_code", strConsultationApply.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("会诊申请单",strConsultationApply.getRecId(), JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrConsultationApply strConsultationApply){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrConsultationApply>()
                .eq("rec_id", strConsultationApply.getRecId())
                .eq("hospital_id", strConsultationApply.getHospitalId())
                .eq("behospital_code", strConsultationApply.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrConsultationApplyVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrConsultationApplyVO strConsultationApplyVO:list) {
                if("".equals(strConsultationApplyVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strConsultationApplyVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strConsultationApplyVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrConsultationApply> strConsultationApplyList= Lists.newArrayList();
            strConsultationApplyList= BeanUtil.listCopyTo(list,StrConsultationApply.class);
            execute(strConsultationApplyList);

            return RespDTO.onSuc(strConsultationApplyList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrConsultationApply> list){
        try {
            List<StrConsultationApply> addE = Lists.newLinkedList();
            List<StrConsultationApply> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrConsultationApply strConsultationApply = this.getOne(new QueryWrapper<StrConsultationApply>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strConsultationApply != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strConsultationApplyService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-会诊申请单","", JSON.toJSONString(list),"",e.getMessage());
        }
    }

}
