package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrDeathDiscussion;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrDeathDiscussionServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrDeathDiscussionVO;
import com.diagbot.vo.str.StrDeathDiscussionVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrDeathDiscussionFacade extends StrDeathDiscussionServiceImpl {
    @Autowired
    @Qualifier("strDeathDiscussionServiceImpl")
    private StrDeathDiscussionServiceImpl strDeathDiscussionService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strDeathDiscussion");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    //死亡病例讨论记录
    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrDeathDiscussion strDeathDiscussion=new StrDeathDiscussion();
        try {
            AStrDeathDiscussionVO strDeathDiscussionVO=(AStrDeathDiscussionVO) MapUtil.mapToObject(entityMap, AStrDeathDiscussionVO.class);
            BeanUtils.copyProperties(strDeathDiscussionVO,strDeathDiscussion);
            strDeathDiscussion.setHospitalId(hospitalId);//医院编码

            strDeathDiscussion.setAuditDate(DateUtil.parseDateTime(strDeathDiscussionVO.getAuditDate()));//审核时间
            strDeathDiscussion.setRecDate(DateUtil.parseDateTime(strDeathDiscussionVO.getRecDate()));//记录时间
            strDeathDiscussion.setDeathDate(DateUtil.parseDateTime(strDeathDiscussionVO.getDeathDate()));//死亡时间
            strDeathDiscussion.setDiscussDate(DateUtil.parseDate(strDeathDiscussionVO.getDiscussDate()));//讨论时间
            strDeathDiscussion.setRecordDate(DateUtil.parseDateTime(strDeathDiscussionVO.getRecordDate()));//病历日期

            strDeathDiscussion.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(strDeathDiscussion)){
                strDeathDiscussion.setGmtCreate(new Date());//记录创建时间
                this.save(strDeathDiscussion);
            }else{
                strDeathDiscussion.setGmtModified(new Date());//记录修改时间
                this.update(strDeathDiscussion,new QueryWrapper<StrDeathDiscussion>()
                        .eq("rec_id", strDeathDiscussion.getRecId())
                        .eq("hospital_id", strDeathDiscussion.getHospitalId())
                        .eq("behospital_code", strDeathDiscussion.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("死亡病例讨论记录",strDeathDiscussion.getRecId(), JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrDeathDiscussion strDeathDiscussion){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrDeathDiscussion>()
                .eq("rec_id", strDeathDiscussion.getRecId())
                .eq("hospital_id", strDeathDiscussion.getHospitalId())
                .eq("behospital_code", strDeathDiscussion.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrDeathDiscussionVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrDeathDiscussionVO strDeathDiscussionVO:list) {
                if("".equals(strDeathDiscussionVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strDeathDiscussionVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strDeathDiscussionVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrDeathDiscussion> strDeathDiscussionList= Lists.newArrayList();
            strDeathDiscussionList= BeanUtil.listCopyTo(list,StrDeathDiscussion.class);
            execute(strDeathDiscussionList);

            return RespDTO.onSuc(strDeathDiscussionList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrDeathDiscussion> list){
        try {
            List<StrDeathDiscussion> addE = Lists.newLinkedList();
            List<StrDeathDiscussion> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrDeathDiscussion strDeathDiscussion = this.getOne(new QueryWrapper<StrDeathDiscussion>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strDeathDiscussion != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strDeathDiscussionService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-死亡病例讨论记录","", JSON.toJSONString(list),"",e.getMessage());
        }
    }
}
