package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrDeathNote;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrDeathNoteServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrDeathNoteVO;
import com.diagbot.vo.str.StrDeathNoteVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrDeathNoteFacade extends StrDeathNoteServiceImpl {

    @Autowired
    @Qualifier("strDeathNoteServiceImpl")
    private StrDeathNoteServiceImpl strDeathNoteService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strDeathNote");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    //死亡记录
    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrDeathNote strDeathNote=new StrDeathNote();
        try {
            AStrDeathNoteVO strDeathNoteVO=(AStrDeathNoteVO) MapUtil.mapToObject(entityMap, AStrDeathNoteVO.class);
            BeanUtils.copyProperties(strDeathNoteVO,strDeathNote);
            strDeathNote.setHospitalId(hospitalId);//医院编码

            strDeathNote.setAuditDate(DateUtil.parseDateTime(strDeathNoteVO.getAuditDate()));//审核时间
            strDeathNote.setRecDate(DateUtil.parseDateTime(strDeathNoteVO.getRecDate()));//记录时间
            strDeathNote.setBehospitalDate(DateUtil.parseDateTime(strDeathNoteVO.getBehospitalDate()));//入院日期
            strDeathNote.setDeathDate(DateUtil.parseDate(strDeathNoteVO.getDeathDate()));//死亡日期
            strDeathNote.setRecordDate(DateUtil.parseDateTime(strDeathNoteVO.getRecordDate()));//病历日期

            strDeathNote.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(strDeathNote)){
                strDeathNote.setGmtCreate(new Date());//记录创建时间
                this.save(strDeathNote);
            }else{
                strDeathNote.setGmtModified(new Date());//记录修改时间
                this.update(strDeathNote,new QueryWrapper<StrDeathNote>()
                        .eq("rec_id", strDeathNote.getRecId())
                        .eq("hospital_id", strDeathNote.getHospitalId())
                        .eq("behospital_code", strDeathNote.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("死亡记录", strDeathNote.getRecId(),JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrDeathNote strDeathNote){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrDeathNote>()
                .eq("rec_id", strDeathNote.getRecId())
                .eq("hospital_id", strDeathNote.getHospitalId())
                .eq("behospital_code", strDeathNote.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrDeathNoteVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrDeathNoteVO strDeathNoteVO:list) {
                if("".equals(strDeathNoteVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strDeathNoteVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strDeathNoteVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrDeathNote> strDeathNoteList= Lists.newArrayList();
            strDeathNoteList= BeanUtil.listCopyTo(list,StrDeathNote.class);
            execute(strDeathNoteList);

            return RespDTO.onSuc(strDeathNoteList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrDeathNote> list){
        try {
            List<StrDeathNote> addE = Lists.newLinkedList();
            List<StrDeathNote> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrDeathNote strDeathNote = this.getOne(new QueryWrapper<StrDeathNote>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strDeathNote != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strDeathNoteService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-死亡记录","", JSON.toJSONString(list),"",e.getMessage());
        }
    }
}
