package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrDifficultCase;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrDifficultCaseServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrDifficultCaseVO;
import com.diagbot.vo.str.StrDifficultCaseVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrDifficultCaseFacade extends StrDifficultCaseServiceImpl{

    @Autowired
    @Qualifier("strDifficultCaseServiceImpl")
    private StrDifficultCaseServiceImpl strDifficultCaseService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strDifficultCase");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrDifficultCase medDifficult=new StrDifficultCase();
        try {
            AStrDifficultCaseVO medDifficultVO=(AStrDifficultCaseVO) MapUtil.mapToObject(entityMap, AStrDifficultCaseVO.class);
            BeanUtils.copyProperties(medDifficultVO,medDifficult);
            medDifficult.setHospitalId(hospitalId);//医院编码
            medDifficult.setRecDate(DateUtil.parseDateTime(medDifficultVO.getRecDate()));
            medDifficult.setAuditDate(DateUtil.parseDateTime(medDifficultVO.getAuditDate()));
            medDifficult.setDiscussDate(DateUtil.parseDateTime(medDifficultVO.getDiscussDate()));
            medDifficult.setRecordDate(DateUtil.parseDateTime(medDifficultVO.getRecordDate()));//病历日期

            medDifficult.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(medDifficult)){
                medDifficult.setGmtCreate(new Date());//记录创建时间
                this.save(medDifficult);
            }else{
                medDifficult.setGmtModified(new Date());//记录修改时间
                this.update(medDifficult,new QueryWrapper<StrDifficultCase>()
                        .eq("rec_id", medDifficult.getRecId())
                        .eq("hospital_id", medDifficult.getHospitalId())
                        .eq("behospital_code", medDifficult.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("疑难病例讨论记录",medDifficult.getRecId(), JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrDifficultCase medDifficult){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrDifficultCase>()
                .eq("rec_id", medDifficult.getRecId())
                .eq("hospital_id", medDifficult.getHospitalId())
                .eq("behospital_code", medDifficult.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrDifficultCaseVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrDifficultCaseVO strDifficultCaseVO:list) {
                if("".equals(strDifficultCaseVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strDifficultCaseVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strDifficultCaseVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrDifficultCase> strDifficultCaseList= Lists.newArrayList();
            strDifficultCaseList= BeanUtil.listCopyTo(list,StrDifficultCase.class);
            execute(strDifficultCaseList);

            return RespDTO.onSuc(strDifficultCaseList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrDifficultCase> list){
        try {
            List<StrDifficultCase> addE = Lists.newLinkedList();
            List<StrDifficultCase> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrDifficultCase strDifficultCase = this.getOne(new QueryWrapper<StrDifficultCase>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strDifficultCase != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strDifficultCaseService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-死亡记录","", JSON.toJSONString(list),"",e.getMessage());
        }
    }
}
