package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrFirstRecord;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrFirstRecordServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrFirstRecordVO;
import com.diagbot.vo.str.StrFirstRecordVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrFirstRecordFacade extends StrFirstRecordServiceImpl {

    @Autowired
    @Qualifier("strFirstRecordServiceImpl")
    private StrFirstRecordServiceImpl strFirstRecordService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strFirstRecord");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrFirstRecord strFirstRecord=new StrFirstRecord();
        try {
            AStrFirstRecordVO strFirstRecordVO=(AStrFirstRecordVO) MapUtil.mapToObject(entityMap, AStrFirstRecordVO.class);
            BeanUtils.copyProperties(strFirstRecordVO,strFirstRecord);
            strFirstRecord.setHospitalId(hospitalId);//医院编码

            strFirstRecord.setAuditDate(DateUtil.parseDateTime(strFirstRecordVO.getAuditDate()));//审核时间
            strFirstRecord.setRecDate(DateUtil.parseDateTime(strFirstRecordVO.getRecDate()));//记录时间
            strFirstRecord.setRecordDate(DateUtil.parseDateTime(strFirstRecordVO.getRecordDate()));//病历日期

            strFirstRecord.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(strFirstRecord)){
                strFirstRecord.setGmtCreate(new Date());//记录创建时间
                this.save(strFirstRecord);
            }else{
                strFirstRecord.setGmtModified(new Date());//记录修改时间
                this.update(strFirstRecord,new QueryWrapper<StrFirstRecord>()
                        .eq("rec_id", strFirstRecord.getRecId())
                        .eq("hospital_id", strFirstRecord.getHospitalId())
                        .eq("behospital_code", strFirstRecord.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("首次病程录", strFirstRecord.getRecId(),JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrFirstRecord strFirstRecord){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrFirstRecord>()
                .eq("rec_id", strFirstRecord.getRecId())
                .eq("hospital_id", strFirstRecord.getHospitalId())
                .eq("behospital_code", strFirstRecord.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrFirstRecordVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrFirstRecordVO strFirstRecordVO:list) {
                if("".equals(strFirstRecordVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strFirstRecordVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strFirstRecordVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrFirstRecord> strFirstRecordList= Lists.newArrayList();
            strFirstRecordList= BeanUtil.listCopyTo(list,StrFirstRecord.class);
            execute(strFirstRecordList);

            return RespDTO.onSuc(strFirstRecordList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrFirstRecord> list){
        try {
            List<StrFirstRecord> addE = Lists.newLinkedList();
            List<StrFirstRecord> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrFirstRecord strFirstRecord = this.getOne(new QueryWrapper<StrFirstRecord>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strFirstRecord != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strFirstRecordService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-首次病程录","", JSON.toJSONString(list),"",e.getMessage());
        }
    }
}
