package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrLeaveHospital;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrLeaveHospitalServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrLeaveHospitalVO;
import com.diagbot.vo.str.StrLeaveHospitalVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrLeaveHospitalFacade extends StrLeaveHospitalServiceImpl {

    @Autowired
    @Qualifier("strLeaveHospitalServiceImpl")
    private StrLeaveHospitalServiceImpl strLeaveHospitalService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strLeaveHospital");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    //危急值记录
    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrLeaveHospital strLeaveHospital=new StrLeaveHospital();
        try {
            AStrLeaveHospitalVO strLeaveHospitalVO=(AStrLeaveHospitalVO) MapUtil.mapToObject(entityMap, AStrLeaveHospitalVO.class);
            BeanUtils.copyProperties(strLeaveHospitalVO,strLeaveHospital);
            strLeaveHospital.setHospitalId(hospitalId);//医院编码

            strLeaveHospital.setAuditDate(DateUtil.parseDateTime(strLeaveHospitalVO.getAuditDate()));//审核时间
            strLeaveHospital.setRecDate(DateUtil.parseDateTime(strLeaveHospitalVO.getRecDate()));//记录时间
            strLeaveHospital.setBehospitalDate(DateUtil.parseDateTime(strLeaveHospitalVO.getBehospitalDate()));//入院日期
            strLeaveHospital.setLeaveHospitalDate(DateUtil.parseDateTime(strLeaveHospitalVO.getLeaveHospitalDate()));//出院时间
            strLeaveHospital.setRecordDate(DateUtil.parseDateTime(strLeaveHospitalVO.getRecordDate()));//病历日期

            strLeaveHospital.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(strLeaveHospital)){
                strLeaveHospital.setGmtCreate(new Date());//记录创建时间
                this.save(strLeaveHospital);
            }else{
                strLeaveHospital.setGmtModified(new Date());//记录修改时间
                this.update(strLeaveHospital,new QueryWrapper<StrLeaveHospital>()
                        .eq("rec_id", strLeaveHospital.getRecId())
                        .eq("hospital_id", strLeaveHospital.getHospitalId())
                        .eq("behospital_code", strLeaveHospital.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("出院小结",strLeaveHospital.getRecId(), JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrLeaveHospital strLeaveHospital){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrLeaveHospital>()
                .eq("rec_id", strLeaveHospital.getRecId())
                .eq("hospital_id", strLeaveHospital.getHospitalId())
                .eq("behospital_code", strLeaveHospital.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrLeaveHospitalVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrLeaveHospitalVO strLeaveHospitalVO:list) {
                if("".equals(strLeaveHospitalVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strLeaveHospitalVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strLeaveHospitalVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrLeaveHospital> strLeaveHospitalList= Lists.newArrayList();
            strLeaveHospitalList= BeanUtil.listCopyTo(list,StrLeaveHospital.class);
            execute(strLeaveHospitalList);

            return RespDTO.onSuc(strLeaveHospitalList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrLeaveHospital> list){
        try {
            List<StrLeaveHospital> addE = Lists.newLinkedList();
            List<StrLeaveHospital> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrLeaveHospital strLeaveHospital = this.getOne(new QueryWrapper<StrLeaveHospital>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strLeaveHospital != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strLeaveHospitalService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-出院小结","", JSON.toJSONString(list),"",e.getMessage());
        }
    }
}
