package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrOperativeNote;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrOperativeNoteServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrOperativeNoteVO;
import com.diagbot.vo.str.StrOperativeNoteVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrOperativeNoteFacade extends StrOperativeNoteServiceImpl {

    @Autowired
    @Qualifier("strOperativeNoteServiceImpl")
    private StrOperativeNoteServiceImpl strOperativeNoteService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strOperativeNote");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    //手术记录
    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrOperativeNote strOperativeNote=new StrOperativeNote();
        try {
            AStrOperativeNoteVO strOperativeNoteVO=(AStrOperativeNoteVO) MapUtil.mapToObject(entityMap, AStrOperativeNoteVO.class);
            BeanUtils.copyProperties(strOperativeNoteVO,strOperativeNote);
            strOperativeNote.setHospitalId(hospitalId);//医院编码

            strOperativeNote.setAuditDate(DateUtil.parseDateTime(strOperativeNoteVO.getAuditDate()));//审核时间
            strOperativeNote.setRecDate(DateUtil.parseDateTime(strOperativeNoteVO.getRecDate()));//记录时间
            strOperativeNote.setOperationDate(DateUtil.parseDateTime(strOperativeNoteVO.getOperationDate()));//手术日期
            strOperativeNote.setStartTime(DateUtil.parseDate(strOperativeNoteVO.getStartTime()));//开始时间
            strOperativeNote.setEndTime(DateUtil.parseDate(strOperativeNoteVO.getEndTime()));//结束时间
            strOperativeNote.setRecordDate(DateUtil.parseDateTime(strOperativeNoteVO.getRecordDate()));//病历日期

            strOperativeNote.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(strOperativeNote)){
                strOperativeNote.setGmtCreate(new Date());//记录创建时间
                this.save(strOperativeNote);
            }else{
                strOperativeNote.setGmtModified(new Date());//记录修改时间
                this.update(strOperativeNote,new QueryWrapper<StrOperativeNote>()
                        .eq("rec_id", strOperativeNote.getRecId())
                        .eq("hospital_id", strOperativeNote.getHospitalId())
                        .eq("behospital_code", strOperativeNote.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("手术记录", strOperativeNote.getRecId(),JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrOperativeNote strOperativeNote){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrOperativeNote>()
                .eq("rec_id", strOperativeNote.getRecId())
                .eq("hospital_id", strOperativeNote.getHospitalId())
                .eq("behospital_code", strOperativeNote.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrOperativeNoteVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrOperativeNoteVO strOperativeNoteVO:list) {
                if("".equals(strOperativeNoteVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strOperativeNoteVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strOperativeNoteVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrOperativeNote> strOperativeNoteList= Lists.newArrayList();
            strOperativeNoteList= BeanUtil.listCopyTo(list,StrOperativeNote.class);
            execute(strOperativeNoteList);

            return RespDTO.onSuc(strOperativeNoteList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrOperativeNote> list){
        try {
            List<StrOperativeNote> addE = Lists.newLinkedList();
            List<StrOperativeNote> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrOperativeNote strOperativeNote = this.getOne(new QueryWrapper<StrOperativeNote>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strOperativeNote != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strOperativeNoteService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-手术记录","", JSON.toJSONString(list),"",e.getMessage());
        }
    }
}
