package com.diagbot.facade.str;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.StrPreoperativeDiscussion;
import com.diagbot.facade.data.AMedAbnormalInfoFacade;
import com.diagbot.facade.data.ColumnFacade;
import com.diagbot.service.impl.StrPreoperativeDiscussionServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AStrPreoperativeDiscussionVO;
import com.diagbot.vo.str.StrPreoperativeDiscussionVO;
import com.google.common.collect.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AStrPreoperativeDiscussionFacade extends StrPreoperativeDiscussionServiceImpl {

    @Autowired
    @Qualifier("strPreoperativeDiscussionServiceImpl")
    private StrPreoperativeDiscussionServiceImpl strPreoperativeDiscussionService;
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"strPreoperativeDiscussion");

        //Map<String, String> map=columnFacade.getColumn("strConsultationNote");
        if(!flag){
            return;
        }
    }

    //入院记录
    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        StrPreoperativeDiscussion strPreoperativeDiscussion=new StrPreoperativeDiscussion();
        try {
            AStrPreoperativeDiscussionVO strPreoperativeDiscussionVO=(AStrPreoperativeDiscussionVO) MapUtil.mapToObject(entityMap, AStrPreoperativeDiscussionVO.class);
            BeanUtils.copyProperties(strPreoperativeDiscussionVO,strPreoperativeDiscussion);
            strPreoperativeDiscussion.setHospitalId(hospitalId);//医院编码

            strPreoperativeDiscussion.setAuditDate(DateUtil.parseDateTime(strPreoperativeDiscussionVO.getAuditDate()));//审核时间
            strPreoperativeDiscussion.setRecDate(DateUtil.parseDateTime(strPreoperativeDiscussionVO.getRecDate()));//记录时间
            strPreoperativeDiscussion.setDiscussTime(DateUtil.parseDateTime(strPreoperativeDiscussionVO.getDiscussTime()));//讨论时间
            strPreoperativeDiscussion.setRecordDate(DateUtil.parseDateTime(strPreoperativeDiscussionVO.getRecordDate()));//病历日期

            strPreoperativeDiscussion.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(strPreoperativeDiscussion)){
                strPreoperativeDiscussion.setGmtCreate(new Date());//记录创建时间
                this.save(strPreoperativeDiscussion);
            }else{
                strPreoperativeDiscussion.setGmtModified(new Date());//记录修改时间
                this.update(strPreoperativeDiscussion,new QueryWrapper<StrPreoperativeDiscussion>()
                        .eq("rec_id", strPreoperativeDiscussion.getRecId())
                        .eq("hospital_id", strPreoperativeDiscussion.getHospitalId())
                        .eq("behospital_code", strPreoperativeDiscussion.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("术前讨论小结",strPreoperativeDiscussion.getRecId(), JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(StrPreoperativeDiscussion strPreoperativeDiscussion){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<StrPreoperativeDiscussion>()
                .eq("rec_id", strPreoperativeDiscussion.getRecId())
                .eq("hospital_id", strPreoperativeDiscussion.getHospitalId())
                .eq("behospital_code", strPreoperativeDiscussion.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }

    public RespDTO execDealData(List<StrPreoperativeDiscussionVO> list){
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (StrPreoperativeDiscussionVO strPreoperativeDiscussionVO:list) {
                if("".equals(strPreoperativeDiscussionVO.getRecId())) {
                    return RespDTO.onError("请输入记录编号!");
                }else if(strPreoperativeDiscussionVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(strPreoperativeDiscussionVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            //初始、格式化数据
            List<StrPreoperativeDiscussion> strPreoperativeDiscussionList= Lists.newArrayList();
            strPreoperativeDiscussionList= BeanUtil.listCopyTo(list,StrPreoperativeDiscussion.class);
            execute(strPreoperativeDiscussionList);

            return RespDTO.onSuc(strPreoperativeDiscussionList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<StrPreoperativeDiscussion> list){
        try {
            List<StrPreoperativeDiscussion> addE = Lists.newLinkedList();
            List<StrPreoperativeDiscussion> updateE = Lists.newLinkedList();
            if (list != null && list.size() > 0) {
                list.stream().forEach(s -> {
                    StrPreoperativeDiscussion strPreoperativeDiscussion = this.getOne(new QueryWrapper<StrPreoperativeDiscussion>()
                            .eq("rec_id", s.getRecId())
                            .eq("hospital_id", s.getHospitalId())
                            .eq("behospital_code",s.getBehospitalCode()), false);
                    if (strPreoperativeDiscussion != null) {
                        s.setGmtModified(new Date());
                        updateE.add(s);
                    } else {
                        s.setGmtCreate(new Date());
                        addE.add(s);
                    }
                });
            }
            if(addE.size()>0){
                strPreoperativeDiscussionService.saveBatch(addE);
            }
            if(updateE.size()>0){
                this.baseMapper.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("结构化-术前讨论小结","", JSON.toJSONString(list),"",e.getMessage());
        }
    }
}
